/************************************************************************
 * OasisOLC - General / oasis.c					v2.0	*
 * Original author: Levork						*
 * Copyright 1996 by Harvey Gilpin					*
 * Copyright 1997-2001 by George Greer (greerga@circlemud.org)		*
 ************************************************************************/

#include "conf.h"
#include "sysdep.h"

#include "structs.h"
#include "utils.h"
#include "interpreter.h"
#include "comm.h"
#include "db.h"
#include "shop.h"
#include "genolc.h"
#include "genmob.h"
#include "genshp.h"
#include "genzon.h"
#include "genwld.h"
#include "genobj.h"
#include "oasis.h"
#include "screen.h"
#include "dg_olc.h"

const char *nrm, *grn, *cyn, *yel;

/*
 * External data structures.
 */
extern struct obj_data *obj_proto;
extern struct char_data *mob_proto;
extern struct room_data *world;
extern zone_rnum top_of_zone_table;
extern struct zone_data *zone_table;
extern struct descriptor_data *descriptor_list;

/* action editor defines */
extern struct social_messg *soc_mess_list;
extern int top_of_socialt;
void aedit_save_to_disk(struct descriptor_data *d);
void free_action(struct social_messg *action);

extern void hedit_save_to_disk(void);
int find_help_rnum(char *keyword);
extern void hedit_setup_new(struct descriptor_data *d, char *new_key);
extern void hedit_setup_existing(struct descriptor_data *d, int rnum);
extern void free_help(struct help_index_element *help);

/*
 * Internal data structures.
 */
struct olc_scmd_info_t {
  const char *text;
  int con_type;
} olc_scmd_info[] = {
  { "room",	CON_REDIT },
  { "object",	CON_OEDIT },
  { "zone",	CON_ZEDIT },
  { "mobile",	CON_MEDIT },
  { "shop",	CON_SEDIT },
  { "trigger",  CON_TRIGEDIT },
  { "help",     CON_HEDIT },
  { "action",   CON_AEDIT },
  { "\n",	-1	  }
};

/* -------------------------------------------------------------------------- */

/*
 * Only player characters should be using OLC anyway.
 */
void clear_screen(struct descriptor_data *d)
{
  if (PRF_FLAGGED(d->character, PRF_CLS))
    send_to_char(d->character, "[H[J");
}

/* -------------------------------------------------------------------------- */

/*
 * Exported ACMD do_oasis function.
 *
 * This function is the OLC interface.  It deals with all the
 * generic OLC stuff, then passes control to the sub-olc sections.
 */
ACMD(do_oasis)
{
  int number = -1, save = 0, real_num;
  struct descriptor_data *d;
  char *buf3;
  char buf[MAX_STRING_LENGTH];
  char buf1[MAX_STRING_LENGTH];
  char buf2[MAX_STRING_LENGTH];

  /*
   * No screwing around as a mobile.
   */
  if (IS_NPC(ch))
    return;

  /*
   * The command to see what needs to be saved, typically 'olc'.
   */
  if (subcmd == SCMD_OLC_SAVEINFO) {
    do_show_save_list(ch);
    return;
  }

  /*
   * Parse any arguments.
   */
  buf3 = two_arguments(argument, buf1, buf2);
  if (!*buf1) {		/* No argument given. */
    switch (subcmd) {
    case SCMD_OASIS_ZEDIT:
    case SCMD_OASIS_REDIT:
      number = GET_ROOM_VNUM(IN_ROOM(ch));
      break;
    case SCMD_OASIS_TRIGEDIT:
    case SCMD_OASIS_OEDIT:
    case SCMD_OASIS_MEDIT:
    case SCMD_OASIS_SEDIT:
      send_to_char(ch, "Specify a %s VNUM to edit.\r\n", olc_scmd_info[subcmd].text);
      return;
    case SCMD_OASIS_HEDIT:
      send_to_char(ch, "Specify a %s entry to edit.\r\n", olc_scmd_info[subcmd].text);
      return;
    case SCMD_OASIS_AEDIT:
      send_to_char(ch, "Specify an action to edit.\r\n");
      return;
    }
  } else if (!isdigit(*buf1)) {
    if (str_cmp("save", buf1) == 0) {
      save = TRUE;
      if (subcmd == SCMD_OASIS_AEDIT || subcmd == SCMD_OASIS_HEDIT)
           number = 0;
      else
      if (is_number(buf2))
	number = atoi(buf2);
      else if (GET_OLC_ZONE(ch) >= 0) {
	zone_rnum zlok;
	if ((zlok = real_zone(GET_OLC_ZONE(ch))) == NOWHERE)
	  number = -1;
	else
	  number = genolc_zone_bottom(zlok); /* Or .top or in between. */
       }

      if (number < 0) {
	send_to_char(ch, "Save which zone?\r\n");
	return;
      }
      if (subcmd == SCMD_OASIS_TRIGEDIT) {
	send_to_char(ch, "Triggers save automatically.\r\n");
	return;
      }
    }
    else if (subcmd == SCMD_OASIS_AEDIT || subcmd == SCMD_OASIS_HEDIT) number = 0;
    else if (subcmd == SCMD_OASIS_ZEDIT && GET_LEVEL(ch) >= LVL_IMPL) {
      if (str_cmp("new", buf1) || !buf3 || !*buf3)
	send_to_char(ch, "Unknown command, perhaps 'new zone lower-room-number upper-room-number'?\r\n");
      else {
	char sbot[MAX_INPUT_LENGTH], stop[MAX_INPUT_LENGTH];
	room_vnum bottom, top;

	skip_spaces(&buf3);	/* actually, atoi() doesn't care... */
	two_arguments(buf3, sbot, stop);

	number = atoi(buf2);
	bottom = atoi(sbot);
	top = atoi(stop);

	zedit_new_zone(ch, number, bottom, top);
      }
      return;
    } else {
      send_to_char(ch, "Yikes!  Stop that, someone will get hurt!\r\n");
      return;
    }
  }

  /*
   * If a numeric argument was given (like a room number), get it.
   */
  if (number == -1)
    number = atoi(buf1);

  /*
   * Check that whatever it is isn't already being edited.
   */
  for (d = descriptor_list; d; d = d->next)
    if (STATE(d) == olc_scmd_info[subcmd].con_type)
      if (d->olc && OLC_NUM(d) == number) {
       if (subcmd == SCMD_OASIS_AEDIT)
	  sprintf(buf, "Actions are already being editted by %s.\r\n",
	  (CAN_SEE(ch, d->character) ? GET_NAME(d->character) : "someone"));
       else
       if (subcmd == SCMD_OASIS_HEDIT)
	  sprintf(buf, "Help files are already being editted by %s.\r\n",
		  (CAN_SEE(ch, d->character) ? GET_NAME(d->character) : "someone"));
	else
	send_to_char(ch, "That %s is currently being edited by %s.\r\n", olc_scmd_info[subcmd].text, PERS(d->character, ch));
	return;
      }
  d = ch->desc;

  /*
   * Give descriptor an OLC structure.
   */
  if (d->olc) {
    mudlog(BRF, LVL_IMMORT, TRUE, "SYSERR: do_oasis: Player already had olc structure.");
    free(d->olc);
  }
  CREATE(d->olc, struct oasis_olc_data, 1);

  /*
   * Find the zone.
   */
    if (subcmd == SCMD_OASIS_HEDIT && !save)
     OLC_ZNUM(d) = find_help_rnum(buf1);
   else
   if ((OLC_ZNUM(d) = real_zone_by_thing(number)) == -1 && subcmd != SCMD_OASIS_AEDIT) {
     send_to_char(ch, "Sorry, there is no zone for that number!\r\n");
     free(d->olc);
     d->olc = NULL;
     return;
    }

  /*
   * Everyone but IMPLs can only edit zones they have been assigned.
   */
  if ( ( (zone_table[OLC_ZNUM(d)].number != GET_OLC_ZONE(ch) ) || (GET_OLC_ZONE(ch) < 1) ) && (GET_LEVEL(ch) < LVL_IMPL) ) {
    send_to_char(ch, "You do not have permission to edit this zone.\r\n");
    mudlog(BRF, LVL_IMPL, TRUE, "OLC: %s tried to edit zone %d allowed zone %d", GET_NAME(ch), zone_table[OLC_ZNUM(d)].number, GET_OLC_ZONE(ch));
    free(d->olc);
    d->olc = NULL;
    return;
    }
  if (subcmd == SCMD_OASIS_HEDIT && GET_OLC_ZONE(ch) != HEDIT_PERMISSION && GET_LEVEL(ch) < LVL_IMPL) {
      send_to_char(ch, "You do not have permssion to edit help entries.\r\n");
      free(d->olc);
      d->olc = NULL;
      return;
    }
  else if ((!(GET_LEVEL(ch) < LVL_IMPL)) &&
       (!ch->player_specials->saved.olc_zone == AEDIT_PERMISSION)) {
     send_to_char(ch, "You do not have permission to edit actions.\r\n");
     free(d->olc);
     return;
    }

  if (save) {
    const char *type = NULL;

    if (subcmd >= 0 && subcmd <= (int)(sizeof(olc_scmd_info) / sizeof(struct olc_scmd_info_t) - 1))
      type = olc_scmd_info[subcmd].text;
    else {
      send_to_char(ch, "Oops, I forgot what you wanted to save.\r\n");
      return;
    }
    if (subcmd == SCMD_OASIS_HEDIT) {
       send_to_char(ch, "Saving all help entries.\r\n");
       sprintf(buf, "OLC: %s saves help entries.", GET_NAME(ch));
       mudlog(CMP, MAX(LVL_BUILDER, GET_INVIS_LEV(ch)), TRUE, buf);
    } else if (subcmd == SCMD_OASIS_AEDIT) {
       send_to_char(ch, "Saving all actions.\r\n");
       sprintf(buf, "OLC: %s saves all actions", GET_NAME(ch));
       mudlog(CMP, LVL_BUILDER, TRUE, buf);
    } else {
    send_to_char(ch, "Saving all %ss in zone %d.\r\n", type, zone_table[OLC_ZNUM(d)].number);
    mudlog(CMP, MAX(LVL_BUILDER, GET_INVIS_LEV(ch)), TRUE, "OLC: %s saves %s info for zone %d.", GET_NAME(ch), type, zone_table[OLC_ZNUM(d)].number);
    }

    switch (subcmd) {
      case SCMD_OASIS_REDIT: save_rooms(OLC_ZNUM(d)); break;
      case SCMD_OASIS_ZEDIT: save_zone(OLC_ZNUM(d)); break;
      case SCMD_OASIS_OEDIT: save_objects(OLC_ZNUM(d)); break;
      case SCMD_OASIS_MEDIT: save_mobiles(OLC_ZNUM(d)); break;
      case SCMD_OASIS_SEDIT: save_shops(OLC_ZNUM(d)); break;
      case SCMD_OASIS_AEDIT: aedit_save_to_disk(d); break;
      case SCMD_OASIS_HEDIT: hedit_save_to_disk(); break;
    }
    free(d->olc);
    d->olc = NULL;
    return;
  }

  if (subcmd != SCMD_OASIS_AEDIT) OLC_NUM(d) = number;
  /* OLC_NUM(d) = number; */ /* removed for above line - aedit */

    else {
       OLC_NUM(d) = 0;
       OLC_STORAGE(d) = strdup(buf1);
       for (OLC_ZNUM(d) = 0; (OLC_ZNUM(d) <= top_of_socialt); OLC_ZNUM(d)++)  {
 	 if (is_abbrev(OLC_STORAGE(d), soc_mess_list[OLC_ZNUM(d)].command))
 	   break;
       }
       if (OLC_ZNUM(d) > top_of_socialt)  {
 	 if (find_command(OLC_STORAGE(d)) > NOTHING)  {
 	    cleanup_olc(d, CLEANUP_ALL);
 	    send_to_char(ch, "That command already exists.\r\n");
 	    return;
 	 }
 	 sprintf(buf, "Do you wish to add the '%s' action? ", OLC_STORAGE(d));
 	 send_to_char(ch, buf);
 	 OLC_MODE(d) = AEDIT_CONFIRM_ADD;
       }
       else  {
 	 sprintf(buf, "Do you wish to edit the '%s' action? ", soc_mess_list[OLC_ZNUM(d)].command);
 	 send_to_char(ch, buf);
 	 OLC_MODE(d) = AEDIT_CONFIRM_EDIT;
       }
    }

  /*
   * Steal player's descriptor and start up subcommands.
   */
  switch (subcmd) {
  case SCMD_OASIS_TRIGEDIT:
    if ((real_num = real_trigger(number)) != NOTHING)
      trigedit_setup_existing(d, real_num);
    else
      trigedit_setup_new(d);
    STATE(d) = CON_TRIGEDIT;
    break;
  case SCMD_OASIS_AEDIT:
    STATE(d) = CON_AEDIT;
    break;
  case SCMD_OASIS_HEDIT:
     if (OLC_ZNUM(d) < 0)
       hedit_setup_new(d, buf1);
     else
       hedit_setup_existing(d, OLC_ZNUM(d));
     STATE(d) = CON_HEDIT;
     break;
  case SCMD_OASIS_REDIT:
    if ((real_num = real_room(number)) >= 0)
      redit_setup_existing(d, real_num);
    else
      redit_setup_new(d);
    STATE(d) = CON_REDIT;
    break;
  case SCMD_OASIS_ZEDIT:
    if ((real_num = real_room(number)) < 0) {
      send_to_char(ch, "That room does not exist.\r\n");
      free(d->olc);
      d->olc = NULL;
      return;
    }
    zedit_setup(d, real_num);
    STATE(d) = CON_ZEDIT;
    break;
  case SCMD_OASIS_MEDIT:
    if ((real_num = real_mobile(number)) < 0)
      medit_setup_new(d);
    else
      medit_setup_existing(d, real_num);
    STATE(d) = CON_MEDIT;
    break;
  case SCMD_OASIS_OEDIT:
    if ((real_num = real_object(number)) >= 0)
      oedit_setup_existing(d, real_num);
    else
      oedit_setup_new(d);
    STATE(d) = CON_OEDIT;
    break;
  case SCMD_OASIS_SEDIT:
    if ((real_num = real_shop(number)) >= 0)
      sedit_setup_existing(d, real_num);
    else
      sedit_setup_new(d);
    STATE(d) = CON_SEDIT;
    break;
  }
  act("$n starts using OLC.", TRUE, d->character, 0, 0, TO_ROOM);
  SET_BIT(PLR_FLAGS(ch), PLR_WRITING);
  if (subcmd == SCMD_OASIS_AEDIT) {
    mudlog(BRF, LVL_IMMORT, TRUE, "OLC: %s starts editing socials", GET_NAME(ch));
  } else
    mudlog(BRF, LVL_IMMORT, TRUE, "OLC: %s starts editing zone %d allowed zone %d", GET_NAME(ch), zone_table[OLC_ZNUM(d)].number, GET_OLC_ZONE(ch));
}

/*------------------------------------------------------------*\
 Exported utilities
\*------------------------------------------------------------*/

/*
 * Set the colour string pointers for that which this char will
 * see at color level NRM.  Changing the entries here will change
 * the colour scheme throughout the OLC.
 */
void get_char_colors(struct char_data *ch)
{
  nrm = CCNRM(ch, C_NRM);
  grn = CCGRN(ch, C_NRM);
  cyn = CCCYN(ch, C_NRM);
  yel = CCYEL(ch, C_NRM);
}

/*
 * This procedure frees up the strings and/or the structures
 * attatched to a descriptor, sets all flags back to how they
 * should be.
 */
void cleanup_olc(struct descriptor_data *d, byte cleanup_type)
{
  /*
   * Clean up WHAT?
   */
  if (d->olc == NULL)
    return;

  /*. Check for storage -- aedit patch -- M. Scott .*/
  if (OLC_STORAGE(d))
    free(OLC_STORAGE(d));

  /*
    * Check for help.
    */
  if (OLC_HELP(d)) {
    switch (cleanup_type) {
    case CLEANUP_ALL:      free_help(OLC_HELP(d));	break;
    case CLEANUP_STRUCTS:  free(OLC_HELP(d));	break;
    default: /* The caller has screwed up. */	break;
    }
  }

  /*
   * Check for a room. free_room doesn't perform
   * sanity checks, we must be careful here.
   */
  if (OLC_ROOM(d)) {
    switch (cleanup_type) {
    case CLEANUP_ALL:
      free_room(OLC_ROOM(d));
      break;
    case CLEANUP_STRUCTS:
      free(OLC_ROOM(d));
      break;
    default: /* The caller has screwed up. */
      log("SYSERR: cleanup_olc: Unknown type!");
      break;
    }
  }

  /*
   * Check for an existing object in the OLC.  The strings
   * aren't part of the prototype any longer.  They get added
   * with strdup().
   */
  if (OLC_OBJ(d)) {
    free_object_strings(OLC_OBJ(d));
    free(OLC_OBJ(d));
  }

  /*
   * Check for a mob.  free_mobile() makes sure strings are not in
   * the prototype.
   */
  if (OLC_MOB(d))
    free_mobile(OLC_MOB(d));

  /*
   * Check for a zone.  cleanup_type is irrelevant here, free() everything.
   */
  if (OLC_ZONE(d)) {
    free(OLC_ZONE(d)->name);
    free(OLC_ZONE(d)->cmd);
    free(OLC_ZONE(d));
  }

  /*
   * Check for a shop.  free_shop doesn't perform sanity checks, we must
   * be careful here.
   */
  if (OLC_SHOP(d)) {
    switch (cleanup_type) {
    case CLEANUP_ALL:
      free_shop(OLC_SHOP(d));
      break;
    case CLEANUP_STRUCTS:
      free(OLC_SHOP(d));
      break;
    default:
      /* The caller has screwed up but we already griped above. */
      break;
    }
  }

  /*. Check for aedit stuff -- M. Scott */
      if (OLC_ACTION(d))  {
 	switch(cleanup_type)  {
 	 case CLEANUP_ALL:
 	   free_action(OLC_ACTION(d));
 	   break;
 	 case CLEANUP_STRUCTS:
 	   free(OLC_ACTION(d));
 	   break;
 	 default:
 	   /* Caller has screwed up */
 	   break;
 	}
      }

  /*
   * Restore descriptor playing status.
   */
  if (d->character) {
    REMOVE_BIT(PLR_FLAGS(d->character), PLR_WRITING);
    STATE(d) = CON_PLAYING;
    act("$n stops using OLC.", TRUE, d->character, NULL, NULL, TO_ROOM);
    mudlog(BRF, LVL_IMMORT, TRUE, "OLC: %s stops editing zone %d allowed zone %d", GET_NAME(d->character), zone_table[OLC_ZNUM(d)].number, GET_OLC_ZONE(d->character));
  }

  free(d->olc);
  d->olc = NULL;
}
